
# ABSTRACT: DZT test distribution

package DZT::Sample;
use strict;
use warnings;

our $VERSION = 1.2;

sub hi {
    __ 'Hi';
}

sub bye {
    __ 'Bye';
}

=head1 Copyright and License

This software is copyright (c) 2012 by David E. Wheeler.

This is free software; you can redistribute it and/or modify it under the same
terms as the Perl 5 programming language system itself.

=cut
